@@ -119,6 +119,26 @@ bool CacheManager::isCached(const QByteArray& expectedHash) const
     return result;
 }
 
+bool CacheManager::hasPotentialCache(const QByteArray& expectedHash) const
+{
+    QMutexLocker locker(&mutex_);
+    // Check if we have a potential cache match (hash matches, file exists)
+    // Does NOT require verification to be complete - used to decide whether to start verification
+    bool result = !expectedHash.isEmpty() && 
+                  status_.cachedHash == expectedHash && 
+                  !status_.cacheFileName.isEmpty() &&
+                  QFile::exists(status_.cacheFileName);
+    
+    if (result) {
+        qDebug() << "Potential cache found for hash:" << expectedHash 
+                 << "file:" << status_.cacheFileName
+                 << "verified:" << status_.verificationComplete
+                 << "valid:" << status_.isValid;
+    }
+    
+    return result;
+}
+
 QString CacheManager::getCacheFilePath(const QByteArray& expectedHash) const
 {
     QMutexLocker locker(&mutex_);
@@ -227,7 +247,10 @@ void CacheManager::startVerification(const QByteArray& expectedHash)
             cacheFileName = status.cacheFileName;
             hashToVerify = expectedHash; // For custom cache files, verify against expected hash
         } else {
-            cacheFileName = getDefaultCacheFilePath();
+            // Use the stored cache file path if available (loaded from settings),
+            // otherwise use the default path. This ensures we verify the actual
+            // cache file that exists, not a path that might differ due to app name changes.
+            cacheFileName = status.cacheFileName.isEmpty() ? getDefaultCacheFilePath() : status.cacheFileName;
             // For regular cache files, verify against the stored compressed hash (cache file contains compressed data)
             hashToVerify = status.cacheFileHash.isEmpty() ? expectedHash : status.cacheFileHash;
         }
