@@ -5,6 +5,7 @@
 
 #include "downloadextractthread.h"
 #include "config.h"
+#include "platformquirks.h"
 #include "systemmemorymanager.h"
 #include "dependencies/drivelist/src/drivelist.hpp"
 #include "dependencies/mountutils/src/mountutils.hpp"
@@ -20,7 +21,6 @@
 #include <QProcess>
 #include <QTemporaryDir>
 #include <QDebug>
-#include <QtConcurrent/qtconcurrentrun.h>
 #include <QElapsedTimer>
 
 #ifdef Q_OS_WIN
@@ -63,7 +63,6 @@ DownloadExtractThread::DownloadExtractThread(const QByteArray &url, const QByteA
       _ethreadStarted(false),
       _isImage(true), 
       _inputHash(OSLIST_HASH_ALGORITHM), 
-      _writeThreadStarted(false),
       _progressStarted(false),
       _lastProgressTime(0),
       _lastEmittedDlNow(0),
@@ -73,7 +72,6 @@ DownloadExtractThread::DownloadExtractThread(const QByteArray &url, const QByteA
       _bytesDecompressed(0),
       _downloadComplete(false),
       _totalDecompressionMs(0),
-      _totalWriteWaitMs(0),
       _totalRingBufferWaitMs(0),
       _bytesReadFromRingBuffer(0)
 {
@@ -87,18 +85,27 @@ DownloadExtractThread::DownloadExtractThread(const QByteArray &url, const QByteA
     _ringBuffer = std::make_unique<RingBuffer>(numSlots, inputBufferSize, pageSize);
     
     // Create ring buffer for decompress -> write path (decompressed data)
-    // Uses 4 slots to provide enough pipeline slack:
-    // - Slot N: being decompressed into
-    // - Slot N-1: write in progress, hash computation running
-    // - Slot N-2: hash being finalized
-    // - Slot N-3: available for reuse
-    // This eliminates the race condition where a buffer was reused before its hash completed
-    static const size_t WRITE_RING_BUFFER_SLOTS = 4;
-    _writeRingBuffer = std::make_unique<RingBuffer>(WRITE_RING_BUFFER_SLOTS, _writeBufferSize, pageSize);
+    // With zero-copy async I/O, ring buffer slots serve as async I/O buffers.
+    // Size based on optimal queue depth (calculated from available memory) plus headroom.
+    // This avoids over-allocation while still allowing decompression to stay ahead.
+    int optimalDepth = SystemMemoryManager::instance().getOptimalAsyncQueueDepth(_writeBufferSize);
+    
+    // Add headroom: 2x optimal depth or optimal + 64, whichever is larger
+    // This allows decompression to buffer ahead while async writes drain
+    int writeRingBufferDepth = qMax(optimalDepth * 2, optimalDepth + 64);
+    
+    // Cap to a reasonable maximum to prevent excessive memory usage
+    // 512 slots @ 8MB = 4GB, which is reasonable for high-memory systems
+    static const int MAX_RING_BUFFER_DEPTH = 512;
+    writeRingBufferDepth = qMin(writeRingBufferDepth, MAX_RING_BUFFER_DEPTH);
+    
+    size_t writeRingBufferSlots = static_cast<size_t>(writeRingBufferDepth + 4);
+    _writeRingBuffer = std::make_unique<RingBuffer>(writeRingBufferSlots, _writeBufferSize, pageSize);
     
     qDebug() << "Using buffer size:" << _writeBufferSize << "bytes with page size:" << pageSize << "bytes";
     qDebug() << "Ring buffer:" << RING_BUFFER_SLOTS << "slots of" << inputBufferSize << "bytes";
-    qDebug() << "Write ring buffer:" << WRITE_RING_BUFFER_SLOTS << "slots of" << _writeBufferSize << "bytes";
+    qDebug() << "Write ring buffer:" << writeRingBufferSlots << "slots of" << _writeBufferSize << "bytes"
+             << "(optimal depth:" << optimalDepth << ", max configurable:" << writeRingBufferDepth << ")";
 }
 
 DownloadExtractThread::~DownloadExtractThread()
@@ -111,6 +118,13 @@ DownloadExtractThread::~DownloadExtractThread()
         _extractThread->terminate();
     }
     
+    // Wait for any pending async writes before destroying ring buffers
+    // The async completion callbacks reference the ring buffer, so we must
+    // ensure they've all completed before destruction
+    if (_file && _file->IsAsyncIOSupported()) {
+        _file->WaitForPendingWrites();
+    }
+    
     // Ring buffer destructors handle memory cleanup
     _writeRingBuffer.reset();
     _ringBuffer.reset();
@@ -167,14 +181,14 @@ void DownloadExtractThread::_emitProgressUpdate()
     
     quint64 currentDlNow = this->dlNow();
     quint64 currentDlTotal = this->dlTotal();
+    quint64 currentExtractTotal = this->extractTotal();
     quint64 currentVerifyNow = this->verifyNow();
     quint64 currentVerifyTotal = this->verifyTotal();
     quint64 currentDecompressNow = _bytesDecompressed.load();
     quint64 currentWriteNow = this->bytesWritten();
     
-    // For decompressed images, the total is the extract size (uncompressed)
-    // We use the same total for both decompress and write since they're the same data
-    quint64 decompressTotal = currentDlTotal > 0 ? currentDlTotal : 0;  // Will be updated by caller
+    // For write progress, use extract size (uncompressed) if set, otherwise fall back to download size
+    quint64 writeTotal = currentExtractTotal > 0 ? currentExtractTotal : currentDlTotal;
     
     // Only emit signals if values have changed
     if (currentDlNow != _lastEmittedDlNow || (currentDlTotal > 0 && _lastEmittedDlNow == 0)) {
@@ -184,12 +198,12 @@ void DownloadExtractThread::_emitProgressUpdate()
     
     if (currentDecompressNow != _lastEmittedDecompressNow) {
         _lastEmittedDecompressNow = currentDecompressNow;
-        emit decompressProgressChanged(currentDecompressNow, decompressTotal);
+        emit decompressProgressChanged(currentDecompressNow, writeTotal);
     }
     
     if (currentWriteNow != _lastEmittedWriteNow) {
         _lastEmittedWriteNow = currentWriteNow;
-        emit writeProgressChanged(currentWriteNow, decompressTotal);
+        emit writeProgressChanged(currentWriteNow, writeTotal);
     }
     
     if (currentVerifyNow != _lastLocalVerifyNow || (currentVerifyTotal > 0 && _lastLocalVerifyNow == 0)) {
@@ -307,17 +321,13 @@ void DownloadExtractThread::extractImageRun()
         // Emit image extraction setup event (archive opened and header read)
         emit eventImageExtraction(static_cast<quint32>(extractionTimer.elapsed()), true);
 
-        // Timers for pipeline instrumentation
+        // Timer for pipeline instrumentation
         QElapsedTimer decompressTimer;
-        QElapsedTimer writeWaitTimer;
-        
-        // Track the previous write slot so we can release it after the write completes
-        RingBuffer::Slot* previousWriteSlot = nullptr;
         
         while (true)
         {
             // Acquire a slot from the write ring buffer
-            // This blocks if all slots are in use (back-pressure from slow writes)
+            // This blocks if all slots are in use (back-pressure from slow writes or async I/O)
             RingBuffer::Slot* slot = _writeRingBuffer->acquireWriteSlot(100);
             while (!slot && !_cancelled && !_writeRingBuffer->isCancelled()) {
                 slot = _writeRingBuffer->acquireWriteSlot(100);
@@ -365,55 +375,47 @@ void DownloadExtractThread::extractImageRun()
             // Emit progress updates during extraction
             _emitProgressUpdate();
 
-            if (_writeThreadStarted)
-            {
-                // Time waiting for previous write to complete
-                writeWaitTimer.start();
-                bool writeResult = _writeFuture.result();
-                _totalWriteWaitMs.fetch_add(static_cast<quint64>(writeWaitTimer.elapsed()));
-                
-                // Previous write is complete (including hash), release its slot
-                if (previousWriteSlot) {
-                    _writeRingBuffer->releaseReadSlot(previousWriteSlot);
-                    previousWriteSlot = nullptr;
-                }
-                
-                if (!writeResult)
-                {
-                    // Release current slot before returning
-                    _writeRingBuffer->releaseReadSlot(slot);
-                    if (!_cancelled)
-                    {
-                        _onWriteError();
-                    }
-                    archive_read_free(a);
-                    return;
-                }
-            }
-
-            // Remember this slot so we can release it after the write completes
-            previousWriteSlot = slot;
+            // Create a completion callback that releases the ring buffer slot
+            // This enables ZERO-COPY async I/O: the slot stays valid until the
+            // async write truly completes, then is returned to the pool.
+            // Capture slot and buffer pointers by value for the callback.
+            RingBuffer* ringBuf = _writeRingBuffer.get();
+            RingBuffer::Slot* slotToRelease = slot;
+            DownloadThread::WriteCompleteCallback releaseCallback = [ringBuf, slotToRelease]() {
+                ringBuf->releaseReadSlot(slotToRelease);
+            };
             
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-            _writeFuture = QtConcurrent::run(&DownloadThread::_writeFile, static_cast<DownloadThread*>(this), slot->data, static_cast<size_t>(size));
-#else
-            _writeFuture = QtConcurrent::run(static_cast<DownloadThread*>(this), &DownloadThread::_writeFile, slot->data, static_cast<size_t>(size));
-#endif
-            _writeThreadStarted = true;
-        }
-
-        if (_writeThreadStarted) {
-            _writeFuture.waitForFinished();
-            // Release the final slot
-            if (previousWriteSlot) {
-                _writeRingBuffer->releaseReadSlot(previousWriteSlot);
-                previousWriteSlot = nullptr;
+            // IMPORTANT: Call _writeFile directly from extraction thread instead of via
+            // QtConcurrent::run(). Using the thread pool causes deadlock when async I/O
+            // is enabled: _writeFile waits for previous hash computation, but hash runs
+            // in the same thread pool. With many queued _writeFile calls, all pool threads
+            // block waiting for hashes that can't run (no available threads).
+            //
+            // With async I/O, _writeFile returns quickly after queuing the I/O operation,
+            // so running it synchronously in the extraction thread doesn't block progress.
+            // The actual I/O happens asynchronously via io_uring/IOCP.
+            bool writeOk = _writeFile(slot->data, static_cast<size_t>(size), releaseCallback) > 0;
+            if (!writeOk && !_cancelled) {
+                // Wait for pending async writes before cleanup
+                if (_file && _file->IsAsyncIOSupported()) {
+                    _file->WaitForPendingWrites();
+                }
+                _onWriteError();
+                archive_read_free(a);
+                return;
             }
         }
+
         _writeComplete();
     }
     catch (exception &e)
     {
+        // Wait for pending async writes before cleanup
+        // Their callbacks reference the ring buffer, so we must wait
+        if (_file && _file->IsAsyncIOSupported()) {
+            _file->WaitForPendingWrites();
+        }
+        
         if (!_cancelled)
         {
             // Fatal error
@@ -429,17 +431,16 @@ void DownloadExtractThread::extractImageRun()
     emit eventPipelineDecompressionTime(
         static_cast<quint32>(_totalDecompressionMs.load()),
         _bytesDecompressed.load());
-    emit eventPipelineWriteWaitTime(
-        static_cast<quint32>(_totalWriteWaitMs.load()),
-        bytesWritten());
     emit eventPipelineRingBufferWaitTime(
         static_cast<quint32>(_totalRingBufferWaitMs.load()),
         _bytesReadFromRingBuffer.load());
     
     qDebug() << "Pipeline timing summary:"
              << "decompress=" << _totalDecompressionMs.load() << "ms"
-             << "(ring_wait=" << _totalRingBufferWaitMs.load() << "ms)"
-             << "write_wait=" << _totalWriteWaitMs.load() << "ms";
+             << "(ring_wait=" << _totalRingBufferWaitMs.load() << "ms)";
+    
+    // Emit detailed write timing breakdown for hypothesis testing
+    _emitWriteTimingStats();
     
     // Log and emit write ring buffer statistics
     if (_writeRingBuffer) {
@@ -478,7 +479,8 @@ void DownloadExtractThread::extractMultiFileRun()
 {
     QString folder;
     QStringList filesExtracted, dirExtracted;
-    QByteArray devlower = _filename.toLower();
+    // Use canonical path for comparison since drivelist returns /dev/disk, not /dev/rdisk
+    QByteArray canonicalDevice = PlatformQuirks::getEjectDevicePath(_filename).toLower().toUtf8();
 
     /* See if OS auto-mounted the device */
     for (int tries = 0; tries < 3; tries++)
@@ -487,7 +489,7 @@ void DownloadExtractThread::extractMultiFileRun()
         auto l = Drivelist::ListStorageDevices();
         for (const auto& i : l)
         {
-            if (QByteArray::fromStdString(i.device).toLower() == devlower && i.mountpoints.size() == 1)
+            if (QByteArray::fromStdString(i.device).toLower() == canonicalDevice && i.mountpoints.size() == 1)
             {
                 folder = QByteArray::fromStdString(i.mountpoints.front());
                 break;
@@ -697,7 +699,9 @@ void DownloadExtractThread::extractMultiFileRun()
 
     if (_ejectEnabled)
     {
-        eject_disk(_filename.constData());
+        // Use canonical device path for eject (e.g., /dev/disk on macOS, not rdisk)
+        QString ejectPath = PlatformQuirks::getEjectDevicePath(_filename);
+        eject_disk(ejectPath.toLocal8Bit().constData());
     }
 }
 
@@ -864,69 +868,8 @@ void DownloadExtractThread::_pushQueue(const char *data, size_t len)
     }
 }
 
-bool DownloadExtractThread::_verify()
+void DownloadExtractThread::_onVerifyProgress()
 {
-    qDebug() << "DownloadExtractThread::_verify() called (child class implementation with progress updates)";
-    _lastVerifyNow = 0;
-    _verifyTotal = _file->Tell();
-    
-    // Use adaptive buffer size based on file size and system memory for optimal verification performance
-    size_t verifyBufferSize = SystemMemoryManager::instance().getAdaptiveVerifyBufferSize(_verifyTotal);
-    char *verifyBuf = (char *) qMallocAligned(verifyBufferSize, 4096);
-    
-    QElapsedTimer t1;
-    t1.start();
-    
-    qDebug() << "Post-write verification using" << verifyBufferSize/1024 << "KB buffer for" 
-             << _verifyTotal/(1024*1024) << "MB image";
-
-    // Platform-specific optimization for sequential read verification
-    // Invalidates cache and enables read-ahead hints
-    _file->PrepareForSequentialRead(0, _verifyTotal);
-
-    if (!_firstBlock)
-    {
-        _file->Seek(0);
-    }
-    else
-    {
-        _verifyhash.addData(_firstBlock, _firstBlockSize);
-        _file->Seek(_firstBlockSize);
-        _lastVerifyNow += _firstBlockSize;
-    }
-
-    while (_verifyEnabled && _lastVerifyNow < _verifyTotal && !_cancelled)
-    {
-        size_t bytes_to_read = qMin((qint64) verifyBufferSize, (qint64) (_verifyTotal-_lastVerifyNow));
-        size_t lenRead = 0;
-        rpi_imager::FileError read_result = _file->ReadSequential(reinterpret_cast<std::uint8_t*>(verifyBuf), bytes_to_read, lenRead);
-        if (read_result != rpi_imager::FileError::kSuccess)
-        {
-            DownloadThread::_onDownloadError(tr("Error reading from storage.<br>"
-                                                "SD card may be broken."));
-            qFreeAligned(verifyBuf);
-            return false;
-        }
-
-        _verifyhash.addData(verifyBuf, static_cast<qint64>(lenRead));
-        _lastVerifyNow += static_cast<qint64>(lenRead);
-        
-        // Emit progress updates during verification
-        _emitProgressUpdate();
-    }
-    qFreeAligned(verifyBuf);
-
-    qDebug() << "Verify hash:" << _verifyhash.result().toHex();
-    qDebug() << "Verify done in" << t1.elapsed() / 1000.0 << "seconds";
-
-    if (_verifyhash.result() == _writehash.result() || !_verifyEnabled || _cancelled)
-    {
-        return true;
-    }
-    else
-    {
-        DownloadThread::_onDownloadError(tr("Verifying write failed. Contents of SD card is different from what was written to it."));
-    }
-
-    return false;
+    // Emit progress updates during verification
+    _emitProgressUpdate();
 }
